/* ------------------------------------------------------------------------- */
/*
 *  test_combine.cpp
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Tue 12 Jan 2010 06:10:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <exception>
#include <string>
#include "clx/combine.h"
#include "clx/mpl/combine.h"

/* ------------------------------------------------------------------------- */
//  test_combine
/* ------------------------------------------------------------------------- */
class test_combine : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_combine);
	CPPUNIT_TEST(test_normal);
	CPPUNIT_TEST(test_container);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_combine() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_combine() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_normal();
	void test_container();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_combine);

/* ------------------------------------------------------------------------- */
//  test_combine::test_normal
/* ------------------------------------------------------------------------- */
void test_combine::test_normal() {
	char  x1 = 0xa2;
	char  x2 = 0x77;
	char  x3 = 0xff;
	char  x4 = 0x8d;
	short x5 = 0xb59c;
	
	CPPUNIT_ASSERT(clx::combine(x1, x2) == 0xa277);
	CPPUNIT_ASSERT(clx::combine(x1, x5) == 0xa2b59c);
	CPPUNIT_ASSERT(clx::combine(x3, x4, x5) == 0xff8db59c);
	CPPUNIT_ASSERT(clx::combine(x1, x2, x3, x4) == 0xa277ff8d);
	CPPUNIT_ASSERT(clx::combine(x2, x3, x4, x5) == 0xff8db59c);
	CPPUNIT_ASSERT(clx::combine(x5, x4, x3, x2) == 0x9c8dff77);
}

/* ------------------------------------------------------------------------- */
//  test_combine::test_container
/* ------------------------------------------------------------------------- */
void test_combine::test_container() {
	std::vector<char> v;
	
	v.push_back(0x35);
	CPPUNIT_ASSERT(clx::combine(v) == 0x35);
	
	v.push_back(0xf2);
	CPPUNIT_ASSERT(clx::combine(v) == 0x35f2);
	
	v.push_back(0x4a);
	CPPUNIT_ASSERT(clx::combine(v) == 0x35f24a);
	
	v.push_back(0xcb);
	CPPUNIT_ASSERT(clx::combine(v) == 0x35f24acb);
	
	v.push_back(0x28);
	CPPUNIT_ASSERT(clx::combine(v) == 0xf24acb28);
	
	v.push_back(0);
	CPPUNIT_ASSERT(clx::combine(v) == 0x4acb2800);
	
	char ar[3];
	ar[0] = 0x32;
	ar[1] = 0xbd;
	ar[2] = 0xe5;
	CPPUNIT_ASSERT(clx::combine(ar) == 0x32bde5);
}
