/* ------------------------------------------------------------------------- */
/*
 *  test_filetype.cpp
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Mon 11 Jan 2010 04:17:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <exception>
#include <string>
#include "clx/filetype.h"

/* ------------------------------------------------------------------------- */
//  test_filetype
/* ------------------------------------------------------------------------- */
class test_filetype : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_filetype);
	CPPUNIT_TEST(test_bmp);
	CPPUNIT_TEST(test_jpg);
	CPPUNIT_TEST(test_gif);
	CPPUNIT_TEST(test_png);
	CPPUNIT_TEST(test_wmf);
	CPPUNIT_TEST(test_emf);
	CPPUNIT_TEST(test_pdf);
	CPPUNIT_TEST(test_doc);
	CPPUNIT_TEST(test_xls);
	CPPUNIT_TEST(test_ppt);
	CPPUNIT_TEST(test_docx);
	CPPUNIT_TEST(test_xlsx);
	CPPUNIT_TEST(test_pptx);
	CPPUNIT_TEST(test_ttf);
	CPPUNIT_TEST(test_ttc);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_filetype() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_filetype() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_bmp();
	void test_jpg();
	void test_gif();
	void test_png();
	void test_wmf();
	void test_emf();
	void test_pdf();
	void test_doc();
	void test_xls();
	void test_ppt();
	void test_docx();
	void test_xlsx();
	void test_pptx();
	void test_ttf();
	void test_ttc();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_filetype);

/* ------------------------------------------------------------------------- */
//  test_filetype::test_bmp
/* ------------------------------------------------------------------------- */
void test_filetype::test_bmp() {
	try {
		std::string path("samples/sample.bmp");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_bmp(path));
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_bmp(path, clx::bmptype::win));
		CPPUNIT_ASSERT_MESSAGE(path, !clx::is_bmp(path, clx::bmptype::os2));
		CPPUNIT_ASSERT(!clx::is_bmp("samples/sample.jpg"));
		CPPUNIT_ASSERT(!clx::is_bmp("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_jpg
/* ------------------------------------------------------------------------- */
void test_filetype::test_jpg() {
	try {
		std::string path("samples/sample.jpg");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_jpg(path));
		CPPUNIT_ASSERT(!clx::is_jpg("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_jpg("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_gif
/* ------------------------------------------------------------------------- */
void test_filetype::test_gif() {
	try {
		std::string path("samples/sample.gif");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_gif(path));
		CPPUNIT_ASSERT(!clx::is_gif("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_gif("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_png
/* ------------------------------------------------------------------------- */
void test_filetype::test_png() {
	try {
		std::string path("samples/sample.png");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_png(path));
		CPPUNIT_ASSERT(!clx::is_png("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_png("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_wmf
/* ------------------------------------------------------------------------- */
void test_filetype::test_wmf() {
	try {
		std::string path("samples/sample_apm.wmf");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_wmf(path, true));
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_wmf(path, false));
		
		path = "samples/sample.wmf";
		CPPUNIT_ASSERT_MESSAGE(path, !clx::is_wmf(path, true));
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_wmf(path, false));
		
		CPPUNIT_ASSERT(!clx::is_wmf("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_wmf("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_emf
/* ------------------------------------------------------------------------- */
void test_filetype::test_emf() {
	try {
		std::string path("samples/sample.emf");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_emf(path));
		CPPUNIT_ASSERT(!clx::is_emf("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_emf("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_pdf
/* ------------------------------------------------------------------------- */
void test_filetype::test_pdf() {
	try {
		std::string path("samples/sample.pdf");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_pdf(path));
		CPPUNIT_ASSERT(!clx::is_pdf("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_pdf("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_doc
/* ------------------------------------------------------------------------- */
void test_filetype::test_doc() {
	try {
		std::string path("samples/sample.doc");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_doc(path));
		CPPUNIT_ASSERT(!clx::is_doc("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_doc("samples/sample.xls"));
		CPPUNIT_ASSERT(!clx::is_doc("samples/sample.ppt"));
		CPPUNIT_ASSERT(!clx::is_doc("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_xls
/* ------------------------------------------------------------------------- */
void test_filetype::test_xls() {
	try {
		std::string path("samples/sample.xls");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_xls(path));
		CPPUNIT_ASSERT(!clx::is_xls("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_xls("samples/sample.doc"));
		CPPUNIT_ASSERT(!clx::is_xls("samples/sample.ppt"));
		CPPUNIT_ASSERT(!clx::is_xls("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_ppt
/* ------------------------------------------------------------------------- */
void test_filetype::test_ppt() {
	try {
		std::string path("samples/sample.ppt");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_ppt(path));
		CPPUNIT_ASSERT(!clx::is_ppt("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_ppt("samples/sample.doc"));
		CPPUNIT_ASSERT(!clx::is_ppt("samples/sample.xls"));
		CPPUNIT_ASSERT(!clx::is_ppt("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_docx
/* ------------------------------------------------------------------------- */
void test_filetype::test_docx() {
	try {
		std::string path("samples/sample.docx");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_docx(path));
		CPPUNIT_ASSERT(!clx::is_docx("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_docx("samples/sample.xlsx"));
		CPPUNIT_ASSERT(!clx::is_docx("samples/sample.pptx"));
		CPPUNIT_ASSERT(!clx::is_docx("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_xlsx
/* ------------------------------------------------------------------------- */
void test_filetype::test_xlsx() {
	try {
		std::string path("samples/sample.xlsx");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_xlsx(path));
		CPPUNIT_ASSERT(!clx::is_xlsx("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_xlsx("samples/sample.docx"));
		CPPUNIT_ASSERT(!clx::is_xlsx("samples/sample.pptx"));
		CPPUNIT_ASSERT(!clx::is_xlsx("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_pptx
/* ------------------------------------------------------------------------- */
void test_filetype::test_pptx() {
	try {
		std::string path("samples/sample.pptx");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_pptx(path));
		CPPUNIT_ASSERT(!clx::is_pptx("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_pptx("samples/sample.docx"));
		CPPUNIT_ASSERT(!clx::is_pptx("samples/sample.xlsx"));
		CPPUNIT_ASSERT(!clx::is_pptx("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_ttf
/* ------------------------------------------------------------------------- */
void test_filetype::test_ttf() {
	try {
		std::string path("samples/arial.ttf");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_ttf(path));
		CPPUNIT_ASSERT(!clx::is_ttf("samples/msgothic.ttc"));
		CPPUNIT_ASSERT(!clx::is_ttf("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_ttf("samples/sample.doc"));
		CPPUNIT_ASSERT(!clx::is_ttf("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filetype::test_ttc
/* ------------------------------------------------------------------------- */
void test_filetype::test_ttc() {
	try {
		std::string path("samples/msgothic.ttc");
		CPPUNIT_ASSERT_MESSAGE(path, clx::is_ttc(path));
		CPPUNIT_ASSERT(!clx::is_ttc("samples/arial.ttf"));
		CPPUNIT_ASSERT(!clx::is_ttc("samples/sample.bmp"));
		CPPUNIT_ASSERT(!clx::is_ttc("samples/sample.doc"));
		CPPUNIT_ASSERT(!clx::is_ttc("dummy"));
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}
