/* ------------------------------------------------------------------------- */
/*
 *  test_http.h
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Thu 17 Jun 2010 22:01:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <exception>
#include "clx/http.h"
#include "clx/base64.h"

/* ------------------------------------------------------------------------- */
//  test_http
/* ------------------------------------------------------------------------- */
class test_http : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_http);
	CPPUNIT_TEST(test_head_success);
	CPPUNIT_TEST(test_post_success);
	CPPUNIT_TEST(test_get_success);
	CPPUNIT_TEST(test_get_fail);
	CPPUNIT_TEST(test_get_basic);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_http() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_http() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_head_success();
	void test_post_success();
	void test_get_success();
	void test_get_fail();
	void test_get_basic();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_http);

/* ------------------------------------------------------------------------- */
//  test_http::test_head_success
/* ------------------------------------------------------------------------- */
void test_http::test_head_success() {
	try {
		clx::http h1("clx.cielquis.net", 80);
		CPPUNIT_ASSERT(h1.head("/"));
		CPPUNIT_ASSERT(h1.code() < 400);
		
		clx::http h2("www.google.com", 80);
		CPPUNIT_ASSERT(h2.head("/"));
		CPPUNIT_ASSERT(h2.code() < 400);
		
		clx::http h3("www.yahoo.co.jp", 80);
		CPPUNIT_ASSERT(h3.head("/"));
		CPPUNIT_ASSERT(h3.code() < 400);
		
		clx::http h5("clx.cielquis.net", 80);
		CPPUNIT_ASSERT(h5.head("/download/clx_doc_0.14.2.tar.gz"));
		CPPUNIT_ASSERT(h5.code() < 400);
		
		clx::http h6("cielquis.net", 80);
		CPPUNIT_ASSERT(h6.head("/trash/empty.txt"));
		CPPUNIT_ASSERT(h6.code() < 400);
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_http::test_post_success
/* ------------------------------------------------------------------------- */
void test_http::test_post_success() {
	try {
		std::string data = "data=hello\r\n";
		clx::http h1("cielquis.net", 80);
		CPPUNIT_ASSERT(h1.post("/trash/echo.cgi", data));
		CPPUNIT_ASSERT(h1.code() < 400);
		
		data = "data=hello&non-definitiondata=hello\r\n";
		clx::http h2("cielquis.net", 80);
		CPPUNIT_ASSERT(h2.post("/trash/echo.cgi", data));
		CPPUNIT_ASSERT(h2.code() < 400);
		
		data = "non-definitiondata=hello\r\n";
		clx::http h3("cielquis.net", 80);
		CPPUNIT_ASSERT(h3.post("/trash/echo.cgi", data));
		CPPUNIT_ASSERT(h3.code() < 400);
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_http::test_get_success
/* ------------------------------------------------------------------------- */
void test_http::test_get_success() {
	try {
		clx::http h1("clx.cielquis.net", 80);
		CPPUNIT_ASSERT(h1.get("/"));
		CPPUNIT_ASSERT(h1.code() < 400);
		
		clx::http h2("www.google.com", 80);
		CPPUNIT_ASSERT(h2.get("/"));
		CPPUNIT_ASSERT(h2.code() < 400);
		
		clx::http h3("www.yahoo.co.jp", 80);
		CPPUNIT_ASSERT(h3.get("/"));
		CPPUNIT_ASSERT(h3.code() < 400);
		
		clx::http h5("clx.cielquis.net", 80);
		CPPUNIT_ASSERT(h5.get("/download/clx_doc_0.14.2.tar.gz"));
		CPPUNIT_ASSERT(h5.code() < 400);
		
		// test the case where Content-Length: 0
		clx::http h6("cielquis.net", 80);
		CPPUNIT_ASSERT(h6.get("/trash/empty.txt"));
		CPPUNIT_ASSERT(h6.code() < 400);
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_http::test_get_fail
/* ------------------------------------------------------------------------- */
void test_http::test_get_fail() {
	try {
		clx::http h1("clx.cielquis.net", 80);
		CPPUNIT_ASSERT(h1.get("/404_not_found.html"));
		CPPUNIT_ASSERT_EQUAL(404, h1.code());
		
		clx::http h2("cielquis-dummy.net", 80);
		CPPUNIT_FAIL("dummy url is found!");
	}
	catch (std::exception& e) {
		// OK
	}
}

/* ------------------------------------------------------------------------- */
//  test_http::test_get_basic
/* ------------------------------------------------------------------------- */
void test_http::test_get_basic() {
	try {
		clx::http h1("cielquis.net", 80);
		h1.get("/trash/auth/hello.txt");
		CPPUNIT_ASSERT_EQUAL(401, h1.code());
		
		std::string s = clx::base64::encode("guest:hoge");
		std::string value = "Basic " + s;
		clx::http::head_type header;
		header["Authorization"] = value;
		h1.get("/trash/auth/hello.txt", header);
		CPPUNIT_ASSERT(h1.code() < 400);
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}
