/* ------------------------------------------------------------------------- */
/*
 *  test_ini.cpp
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Fri 22 Jan 2010 16:17:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <exception>
#include <string>
#include <sstream>
#include "clx/ini.h"
#include "clx/filesystem.h"

/* ------------------------------------------------------------------------- */
//  test_ini
/* ------------------------------------------------------------------------- */
class test_ini : public CPPUNIT_NS::TestFixture {
public:
	CPPUNIT_TEST_SUITE(test_ini);
	CPPUNIT_TEST(test_read);
	CPPUNIT_TEST(test_write);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_ini() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_ini() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_read();
	void test_write();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_ini);

/* ------------------------------------------------------------------------- */
//  test_ini::test_read
/* ------------------------------------------------------------------------- */
void test_ini::test_read() {
	std::string s1("samples/test_01.ini");
	try {
		clx::ini v1(s1);
		CPPUNIT_ASSERT(!v1.empty());
		CPPUNIT_ASSERT(v1.size() == 1);
		CPPUNIT_ASSERT(v1[""].size() == 14);
	}
	catch (std::exception& e) {
		std::stringstream ss;
		ss << "1st: " << s1 << ": " << e.what();
		CPPUNIT_FAIL(ss.str());
	}
	
	std::string s2("samples/test_02.ini");
	try {
		clx::ini v2(s2);
		CPPUNIT_ASSERT(!v2.empty());
		CPPUNIT_ASSERT(v2.size() == 3);
		CPPUNIT_ASSERT(v2["Settings"].size() == 14);
		CPPUNIT_ASSERT(v2["Output"].size() == 1);
		CPPUNIT_ASSERT(v2["Debug"].size() == 1);
		CPPUNIT_ASSERT(v2.find("") == v2.end());
	}
	catch (std::exception& e) {
		std::stringstream ss;
		ss << "2nd: " << s2 << ": " << e.what();
		CPPUNIT_FAIL(ss.str());
	}
	
	std::string s3("samples/test_03.ini");
	try {
		clx::ini v3(s3);
		CPPUNIT_ASSERT(!v3.empty());
		CPPUNIT_ASSERT(v3.size() == 3);
		CPPUNIT_ASSERT(v3[""].size() == 14);
		CPPUNIT_ASSERT(v3["Output"].size() == 1);
		CPPUNIT_ASSERT(v3["Debug"].size() == 1);
	}
	catch (std::exception& e) {
		std::stringstream ss;
		ss << "3rd: " << s3 << ": " << e.what();
		CPPUNIT_FAIL(ss.str());
	}
	
	std::string s4("samples/test_04.ini");
	try {
		clx::ini v4(s4);
		CPPUNIT_ASSERT(!v4.empty());
		CPPUNIT_ASSERT(v4.size() == 4);
		CPPUNIT_ASSERT(v4[""].size() == 1);
		CPPUNIT_ASSERT(v4["Settings"].size() == 14);
		CPPUNIT_ASSERT(v4["Output"].size() == 1);
		CPPUNIT_ASSERT(v4["Debug"].size() == 1);
	}
	catch (std::exception& e) {
		std::stringstream ss;
		ss << "4th: " << s4 << ": " << e.what();
		CPPUNIT_FAIL(ss.str());
	}
	
	std::string s5("dummy");
	try {
		clx::ini v5(s5);
		CPPUNIT_FAIL("must not reach the position.");
	}
	catch  (std::exception& e) {
		// catch and release :p
	}
}

/* ------------------------------------------------------------------------- */
//  test_ini::test_write
/* ------------------------------------------------------------------------- */
void test_ini::test_write() {
	std::string in1("samples/test_04.ini");
	std::string out1("samples/test_04_out.ini");
	try {
		clx::ini v1(in1);
		CPPUNIT_ASSERT(!v1.empty());
		CPPUNIT_ASSERT(v1.size() == 4);
		CPPUNIT_ASSERT(v1[""].size() == 1);
		CPPUNIT_ASSERT(v1["Settings"].size() == 14);
		CPPUNIT_ASSERT(v1["Output"].size() == 1);
		CPPUNIT_ASSERT(v1["Debug"].size() == 1);
		
		// add some section, and values.
		v1[""]["OutputTest"] = "test_04_out.ini";
		CPPUNIT_ASSERT(v1[""].size() == 2);
		v1["AdditionalSection"]["Foo"] = "Bar";
		v1.insert("EmptySection");
		
		v1.write(out1);
		clx::ini check(out1);
		CPPUNIT_ASSERT(!check.empty());
		CPPUNIT_ASSERT(check.size() == 6);
		CPPUNIT_ASSERT(check[""].size() == 2);
		CPPUNIT_ASSERT(check["Settings"].size() == 14);
		CPPUNIT_ASSERT(check["Output"].size() == 1);
		CPPUNIT_ASSERT(check["Debug"].size() == 1);
		CPPUNIT_ASSERT(check["AdditionalSection"].size() == 1);
		CPPUNIT_ASSERT(check["EmptySection"].empty());
		std::remove(out1.c_str());
	}
	catch (std::exception& e) {
		if (clx::filesystem::exists(out1)) std::remove(out1.c_str());
		std::stringstream ss;
		ss << "1st: " << in1 << "->" << out1 << ": " << e.what();
		CPPUNIT_FAIL(ss.str());
	}
}
