/* ------------------------------------------------------------------------- */
/*
 *  test_istream_utility.cpp
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Wed 13 Jan 2010 20:44:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <string>
#include <vector>
#include <list>
#include "clx/istream_utility.h"
#include "clx/vstream.h"
#include "clx/combine.h"
#include "clx/mpl.h"

/* ------------------------------------------------------------------------- */
//  test_istream_utility
/* ------------------------------------------------------------------------- */
class test_istream_utility : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_istream_utility);
	CPPUNIT_TEST(test_read);
	CPPUNIT_TEST(test_get);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_istream_utility() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_istream_utility() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_read();
	void test_get();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_istream_utility);

/* ------------------------------------------------------------------------- */
//  test_istream_utility::test_read
/* ------------------------------------------------------------------------- */
void test_istream_utility::test_read() {
	char s1[] = "Hello, world!";
	clx::ivstream v1(s1);
	std::string a1;
	clx::read(v1, a1);
	CPPUNIT_ASSERT(a1.size() == sizeof(s1));
	CPPUNIT_ASSERT(v1.eof());
	CPPUNIT_ASSERT(!v1.bad() && !v1.fail());
	
	std::string s2;
	for (std::size_t i = 0; i < 1000000; ++i) s2 += s1;
	clx::ivstream v2(s2);
	std::vector<char> a2;
	clx::read(v2, a2);
	CPPUNIT_ASSERT(a2.size() == s2.size());
	CPPUNIT_ASSERT(v2.eof());
	CPPUNIT_ASSERT(!v2.bad() && !v2.fail());
	
	std::vector<char> s3;
	for (std::size_t i = 0; i < 10; ++i) {
		s3.push_back('a');
		s3.push_back('b');
		s3.push_back('c');
		s3.push_back('d');
	}
	clx::ivstream v3(s3);
	std::list<std::size_t> a3;
	clx::read(v3, a3);
	CPPUNIT_ASSERT(a3.size() == s3.size() / sizeof(std::size_t));
	std::size_t x3 = clx::mpl::reverse<clx::mpl::combine4<'a', 'b', 'c', 'd'>::value>::value;
	for (std::list<std::size_t>::iterator pos = a3.begin(); pos != a3.end(); ++pos) {
		CPPUNIT_ASSERT(*pos == x3);
	}
	CPPUNIT_ASSERT(v3.eof());
	CPPUNIT_ASSERT(!v3.bad() && !v3.fail());
	
	std::vector<char> s4;
	for (std::size_t i = 0; i < 10; ++i) {
		s4.push_back('a');
		s4.push_back('b');
		s4.push_back('c');
		s4.push_back('d');
	}
	clx::ivstream v4(s4);
	std::list<std::size_t> a4;
	clx::read(v4, a4, clx::endian::big);
	CPPUNIT_ASSERT(a4.size() == s4.size() / sizeof(std::size_t));
	std::size_t x4 = clx::mpl::combine4<'a', 'b', 'c', 'd'>::value;
	for (std::list<std::size_t>::iterator pos = a4.begin(); pos != a4.end(); ++pos) {
		CPPUNIT_ASSERT(*pos == x4);
	}
	CPPUNIT_ASSERT(v4.eof());
	CPPUNIT_ASSERT(!v4.bad() && !v4.fail());
	
	char s5[] = "Hello, world!";
	clx::ivstream v5(s5);
	char a5[64];
	CPPUNIT_ASSERT(clx::read(v5, a5) == sizeof(s5));
	CPPUNIT_ASSERT(v5.eof());
	CPPUNIT_ASSERT(!v5.bad() && !v5.fail());
	
	char s6[] = "Hello, world!";
	clx::ivstream v6(s6);
	char a6[4];
	CPPUNIT_ASSERT(clx::read(v6, a6) == sizeof(a6));
	CPPUNIT_ASSERT(v6.good());
}

/* ------------------------------------------------------------------------- */
//  test_istream_utility::test_get
/* ------------------------------------------------------------------------- */
void test_istream_utility::test_get() {
	std::vector<char> s;
	for (std::size_t i = 0; i < 10; ++i) {
		s.push_back('a');
		s.push_back('b');
		s.push_back('c');
		s.push_back('d');
	}
	clx::ivstream v(s);
	
	char a1 = 0;
	CPPUNIT_ASSERT(clx::get(v, a1));
	CPPUNIT_ASSERT(a1 == 'a');
	CPPUNIT_ASSERT(clx::get(v, a1));
	CPPUNIT_ASSERT(a1 == 'b');
	CPPUNIT_ASSERT(clx::get(v, a1));
	CPPUNIT_ASSERT(a1 == 'c');
	CPPUNIT_ASSERT(clx::get(v, a1));
	CPPUNIT_ASSERT(a1 == 'd');
	
	std::size_t a2 = 0;
	std::size_t x2 = clx::mpl::reverse<clx::mpl::combine4<'a', 'b', 'c', 'd'>::value>::value;
	CPPUNIT_ASSERT(clx::get(v, a2));
	CPPUNIT_ASSERT(a2 == x2);
	
	std::size_t a3 = 0;
	std::size_t x3 = clx::mpl::combine4<'a', 'b', 'c', 'd'>::value;
	CPPUNIT_ASSERT(clx::get(v, a3, clx::endian::big));
	CPPUNIT_ASSERT(a3 == x3);
	
	char a4 = 0;
	while (clx::get(v, a4));
	CPPUNIT_ASSERT(v.eof() && v.fail());
}
