/* ------------------------------------------------------------------------- */
/*
 *  test_mpl.cpp
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Thu 28 Jan 2010 13:57:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <exception>
#include <string>
#include "clx/mpl.h"

/* ------------------------------------------------------------------------- */
//  test_mpl
/* ------------------------------------------------------------------------- */
class test_mpl : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_mpl);
	CPPUNIT_TEST(test_lower_mask);
	CPPUNIT_TEST(test_upper_mask);
	CPPUNIT_TEST(test_combine);
	CPPUNIT_TEST(test_reverse);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_mpl() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_mpl() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_lower_mask();
	void test_upper_mask();
	void test_combine();
	void test_reverse();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_mpl);

/* ------------------------------------------------------------------------- */
//  test_mpl::test_lower_mask
/* ------------------------------------------------------------------------- */
void test_mpl::test_lower_mask() {
	std::size_t x1 = clx::mpl::lower_mask<2>::value;
	CPPUNIT_ASSERT(x1 == 0x03);
	
	std::size_t x2 = clx::mpl::lower_mask<10>::value;
	CPPUNIT_ASSERT(x2 == 0x03ff);
	
	std::size_t x3 = clx::mpl::lower_mask<15>::value;
	CPPUNIT_ASSERT(x3 == 0x7fff);
	
	std::size_t x4 = clx::mpl::lower_mask<16>::value;
	CPPUNIT_ASSERT(x4 == 0xffff);
	
	std::size_t x5 = clx::mpl::lower_mask<31>::value;
	CPPUNIT_ASSERT(x5 == 0x7fffffff);
	
	std::size_t x6 = clx::mpl::lower_mask<2, unsigned char>::value;
	CPPUNIT_ASSERT(x6 == 0x03);
	
	std::size_t x7 = clx::mpl::lower_mask<2, unsigned short>::value;
	CPPUNIT_ASSERT(x7 == 0x03);
}

/* ------------------------------------------------------------------------- */
//  test_mpl::test_upper_mask
/* ------------------------------------------------------------------------- */
void test_mpl::test_upper_mask() {
	std::size_t x1 = clx::mpl::upper_mask<2>::value;
	CPPUNIT_ASSERT(x1 == 0xfffffffc);
	
	std::size_t x2 = clx::mpl::upper_mask<10>::value;
	CPPUNIT_ASSERT(x2 == 0xfffffc00);
	
	std::size_t x3 = clx::mpl::upper_mask<15>::value;
	CPPUNIT_ASSERT(x3 == 0xffff8000);
	
	std::size_t x4 = clx::mpl::upper_mask<16>::value;
	CPPUNIT_ASSERT(x4 == 0xffff0000);
	
	std::size_t x5 = clx::mpl::upper_mask<31>::value;
	CPPUNIT_ASSERT(x5 == 0x80000000);
	
	std::size_t x6 = clx::mpl::upper_mask<2, unsigned char>::value;
	CPPUNIT_ASSERT(x6 == 0xfc);
	
	std::size_t x7 = clx::mpl::upper_mask<2, unsigned short>::value;
	CPPUNIT_ASSERT(x7 == 0xfffc);
}

/* ------------------------------------------------------------------------- */
//  test_mpl::test_combine
/* ------------------------------------------------------------------------- */
void test_mpl::test_combine() {
	std::size_t x1 = clx::mpl::combine2<'l', 's'>::value;
	CPPUNIT_ASSERT(x1 == 0x6c73);
	
	std::size_t x2 = clx::mpl::combine2<0xb29a, 'a'>::value;
	CPPUNIT_ASSERT(x2 == 0xb29a61);
	
	std::size_t x3 = clx::mpl::combine2<0x35f24acb, 'a'>::value;
	CPPUNIT_ASSERT(x3 == 0xf24acb61);
	
	std::size_t x4 = clx::mpl::combine3<'f', 'o', 'o'>::value;
	CPPUNIT_ASSERT(x4 == 0x666f6f);
	
	std::size_t x5 = clx::mpl::combine3<0xb29a, 'l', 's'>::value;
	CPPUNIT_ASSERT(x5 == 0xb29a6c73);
	
	std::size_t x6 = clx::mpl::combine4<'h', 'e', 'l', 'o'>::value;
	CPPUNIT_ASSERT(x6 == 0x68656c6f);
	
	std::size_t x7 = clx::mpl::combine4<'h', 'e', 'l', 0>::value;
	CPPUNIT_ASSERT(x7 == 0x68656c00);
	
	// 4th template-argument is casted into "char" type (0xfe98 -> 0x98).
	std::size_t x8 = clx::mpl::combine4<'h', 'e', 'l', 0xfe98>::value;
	CPPUNIT_ASSERT(x8 == 0x68656c98);
}

/* ------------------------------------------------------------------------- */
//  test_mpl::test_reverse
/* ------------------------------------------------------------------------- */
void test_mpl::test_reverse() {
	std::size_t x1 = clx::mpl::reverse<clx::mpl::combine2<'l', 's'>::value>::value;
	CPPUNIT_ASSERT(x1 == 0x736c0000);
	
	std::size_t x2 = clx::mpl::reverse<clx::mpl::combine2<0xb29a, 'a'>::value>::value;
	CPPUNIT_ASSERT(x2 == 0x619ab200);
	
	std::size_t x3 = clx::mpl::reverse<clx::mpl::combine2<0x35f24acb, 'a'>::value>::value;
	CPPUNIT_ASSERT(x3 == 0x61cb4af2);
	
	std::size_t x4 = clx::mpl::reverse<clx::mpl::combine3<'f', 'o', 'o'>::value>::value;
	CPPUNIT_ASSERT(x4 == 0x6f6f6600);
	
	std::size_t x5 = clx::mpl::reverse<clx::mpl::combine3<0xb29a, 'l', 's'>::value>::value;
	CPPUNIT_ASSERT(x5 == 0x736c9ab2);
	
	std::size_t x6 = clx::mpl::reverse<clx::mpl::combine4<'h', 'e', 'l', 'o'>::value>::value;
	CPPUNIT_ASSERT(x6 == 0x6f6c6568);
	
	std::size_t x7 = clx::mpl::reverse<clx::mpl::combine4<'h', 'e', 'l', 0>::value>::value;
	CPPUNIT_ASSERT(x7 == 0x006c6568);
	
	// 4th template-argument is casted into "char" type (0xfe98 -> 0x98).
	std::size_t x8 = clx::mpl::reverse<clx::mpl::combine4<'h', 'e', 'l', 0xfe98>::value>::value;
	CPPUNIT_ASSERT(x8 == 0x986c6568);
}
