/* ------------------------------------------------------------------------- */
/*
 *  test_tokenizer.cpp
 *
 *  Copyright (c) 2004 - 2009, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Wed 02 Sep 2009 02:02:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <string>
#include "clx/tokenizer.h"

/* ------------------------------------------------------------------------- */
//  test_tokenizer
/* ------------------------------------------------------------------------- */
class test_tokenizer : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_tokenizer);
	CPPUNIT_TEST(test_simple);
	CPPUNIT_TEST(test_csv);
	CPPUNIT_TEST(test_format);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_tokenizer() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_tokenizer() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_simple();
	void test_csv();
	void test_format();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_tokenizer);

/* ------------------------------------------------------------------------- */
//  test_tokenizer::test_simple
/* ------------------------------------------------------------------------- */
void test_tokenizer::test_simple() {
	std::string s1 = "This is, a test.";
	clx::char_separator<char> sep1;
	clx::tokenizer t1(s1, sep1);
	CPPUNIT_ASSERT(t1.size() == 4);
	
	std::string s2 = "This is, a test.";
	clx::char_separator<char> sep2(",");
	clx::tokenizer t2(s2, sep2);
	CPPUNIT_ASSERT(t2.size() == 2);
	
	std::string s3 = "This is, a test.";
	clx::char_separator<char> sep3(".");
	clx::tokenizer t3(s3, sep3);
	CPPUNIT_ASSERT(t3.size() == 1);
	
	std::string s4 = "This is, a test.....";
	clx::char_separator<char> sep4(".", true);
	clx::tokenizer t4(s4, sep4);
	CPPUNIT_ASSERT(t4.size() == 5);
}

/* ------------------------------------------------------------------------- */
//  test_tokenizer::test_csv
/* ------------------------------------------------------------------------- */
void test_tokenizer::test_csv() {
	std::string s1 = "foo,bar,bas";
	clx::escape_separator<char> sep1;
	clx::csv t1(s1, sep1);
	CPPUNIT_ASSERT(t1.size() == 3);
	
	std::string s2 = "\"foo,bar,bas\",\"hoge,fuga\"";
	clx::escape_separator<char> sep2;
	clx::csv t2(s2, sep2);
	CPPUNIT_ASSERT(t2.size() == 2);
	
	std::string s3 = "\"foo,bar,bas\",\"hoge,fuga\",toge";
	clx::escape_separator<char> sep3;
	clx::csv t3(s3, sep3);
	CPPUNIT_ASSERT(t3.size() == 3);
	
	std::string s4 = "\"foo,bar,bas\",\"hoge,fuga\",toge\\,kiku";
	clx::escape_separator<char> sep4;
	clx::csv t4(s4, sep4);
	CPPUNIT_ASSERT(t4.size() == 3);
	
	std::string s5 = "\"foo,bar,bas\",\"hoge,fuga\",toge\\,kiku\\";
	clx::escape_separator<char> sep5;
	clx::csv t5(s5, sep5);
	CPPUNIT_ASSERT(t5.size() == 2);
}

/* ------------------------------------------------------------------------- */
//  test_tokenizer::test_format
/* ------------------------------------------------------------------------- */
void test_tokenizer::test_format() {
	std::string s1 = "Tue Sep  1 22:30:12 JST 2009";
	std::string fmt1 = "%s %s %s %s:%s:%s %s %s";
	clx::format_separator<char> sep1(fmt1);
	clx::strftokenizer t1(s1, sep1);
	CPPUNIT_ASSERT(t1.size() == 8);
	
	std::string s2 = "20090901223012";
	std::string fmt2 = "%4s%2s%2s%2s%2s%2s";
	clx::format_separator<char> sep2(fmt2);
	clx::strftokenizer t2(s2, sep2);
	CPPUNIT_ASSERT(t2.size() == 6);
	
	std::string s3 = "23:30:25.030254  IP  192.168.1.2.100 > 192.168.1.3.200: . ack 12 win 256 <XXX>";
	std::string fmt3 = "%s:%s:%s.%s IP %s > %s: %s ack %s win %s <%s>";
	clx::format_separator<char> sep3(fmt3);
	clx::strftokenizer t3(s3, sep3);
	CPPUNIT_ASSERT(t3.size() == 10);
	
	std::string s4 = "Hello, world!";
	clx::format_separator<char> sep4;
	clx::strftokenizer t4(s4, sep4);
	CPPUNIT_ASSERT(t4.size() == 1);
	
	std::string s5 = "";
	clx::format_separator<char> sep5;
	clx::strftokenizer t5(s5, sep5);
	CPPUNIT_ASSERT(t5.empty());
	
	std::string s6 = "Hello, world!";
	std::string fmt6 = "%s. %s!";
	clx::format_separator<char> sep6(fmt6);
	clx::strftokenizer t6(s6, sep6);
	CPPUNIT_ASSERT(t6.size() == 1);
	
	std::string s7 = "Hello, world!";
	std::string fmt7 = "Hoge%s. %s!";
	clx::format_separator<char> sep7(fmt7);
	clx::strftokenizer t7(s7, sep7);
	CPPUNIT_ASSERT(t7.empty());
	
	std::string s8 = " Hello, world!";
	std::string fmt8 = "%s, %s!";
	clx::format_separator<char> sep8(fmt8);
	clx::strftokenizer t8(s8, sep8);
	CPPUNIT_ASSERT(t8.size() == 2);
	
	std::string s9 = "Hello, world!";
	std::string fmt9 = " %s, %s!";
	clx::format_separator<char> sep9(fmt9);
	clx::strftokenizer t9(s9, sep9);
	CPPUNIT_ASSERT(t9.empty());
}
