/* ------------------------------------------------------------------------- */
/*
 *  test_unzip.cpp
 *
 *  Copyright (c) 2004 - 2009, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Fri 05 Jun 2009 15:13:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <string>
#include "clx/utility.h"
#include "clx/unzip.h"

/* ------------------------------------------------------------------------- */
//  test_unzip
/* ------------------------------------------------------------------------- */
class test_unzip : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_unzip);
	CPPUNIT_TEST(test_open);
	CPPUNIT_TEST(test_read);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_unzip() : CPPUNIT_NS::TestFixture(), path_("samples/clx_doc_0.16.1.zip") {}
	virtual ~test_unzip() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	typedef clx::basic_unzip<char> unzip;
	
	void test_open();
	void test_read();
	
private:
	std::string path_;
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_unzip);

/* ------------------------------------------------------------------------- */
//  test_unzip::test_open
/* ------------------------------------------------------------------------- */
void test_unzip::test_open() {
	clx::unzip suc;
	CPPUNIT_ASSERT_MESSAGE("cannot find input file", suc.open(path_));
	
	clx::unzip failed;
	CPPUNIT_ASSERT_MESSAGE("dummy file is opened!", !failed.open("dummy.zip"));
}

/* ------------------------------------------------------------------------- */
//  test_unzip::test_read
/* ------------------------------------------------------------------------- */
void test_unzip::test_read() {
	clx::unzip in;
	CPPUNIT_ASSERT_MESSAGE("cannot find input file", in.open(path_));
	
	// 1. simple situation.
	for (clx::unzip::iterator pos = in.begin(); pos != in.end(); ++pos) {
		if (pos->path().at(pos->path().size() - 1) == '/') continue; // directory
		
		std::string s;
		clx::read(*pos, s);
		CPPUNIT_ASSERT_MESSAGE(pos->path() + ": failed to read a file", !s.empty());
	}
	
	// 2. little complex
	for (clx::unzip::iterator pos = in.begin(); pos != in.end(); ++pos) {
		// directory
		if (pos->path().at(pos->path().size() - 1) == '/') continue; // directory
		
		std::string s;
		clx::read(*pos, s);
		CPPUNIT_ASSERT_MESSAGE(pos->path() + ": failed to read a file", !s.empty());
		
		clx::unzip::iterator pos2 = in.find("document/index.html");
		CPPUNIT_ASSERT_MESSAGE(pos2->path() + ": failed to find", pos != in.end());
	}
	
	// 3. failed test
	clx::unzip::iterator pos = in.find("dummy.html");
	CPPUNIT_ASSERT_MESSAGE("dummy file is opened!", pos == in.end());
}
